/*
curses-plus-utils, aims to provide a C++-like interface to program with curses (ncurses, pdcurses)
library, and provide a set of more complex controls based on the primitive WINDOW* from curses.

Copyright (C) 2008 Andrius da Costa Ribas <andriusmao@gmail.com>,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

*/


#include <curses.h>
#include <cstdarg>
#include <string>


class WINPLUS
{
	WINDOW* W;
protected:
	bool lastoper; //whether the last operation was successful
	int lastscan; // how many variables were mapped in last scanw-like operation
	void setwpointer(WINDOW *reference); /* apenas heranças podem modificar o ponteiro */
	WINDOW *getwpointer(); /* apenas heranças podem acessar o ponteiro */
	WINPLUS(){} /* para herança de WPLUSMAIN*/
public:
	WINPLUS(int nlines, int ncols, int begin_y, int begin_x);
	
	bool good(); // returns status of last operation
	int scanned(); // how many variables were mapped in last scanw-like operation
	
	int chget(); // equiv. wgetch
	WINPLUS& keypad(bool active); // equiv. keypad
	WINPLUS& noecho(bool active); // equiv. noecho
	WINPLUS& move(int y, int x);
	
	
	// wprintw-like / wscanw-like
	WINPLUS& print(char *mask, ...);
	WINPLUS& vprint(char *mask, va_list args);
	WINPLUS& scan(char *mask, ...);
	WINPLUS& vscan(char *mask, va_list args);
	
	
	// cout-like
	WINPLUS& operator<<(int k);
	WINPLUS& operator<<(float k);
	WINPLUS& operator<<(char k);
	WINPLUS& operator<<(double k);
	WINPLUS& operator<<(char* k);
	WINPLUS& operator<<(std::string k);
	
	
	// cin-like
	WINPLUS& operator>>(int& k);
	WINPLUS& operator>>(float& k);
	WINPLUS& operator>>(char& k);
	WINPLUS& operator>>(double& k);
	WINPLUS& operator>>(char*& k);
	WINPLUS& operator>>(std::string& k);
	
	// wattr- functions (here called a- functions), some names can't be repeated, because they're macros, "c_" stands for "wcolor_"
	WINPLUS& aoff(int attrs);
	WINPLUS& aon(int attrs);
	WINPLUS& aset(int attrs);
	WINPLUS& c_set(short color_pair_number, void* opts);
	WINPLUS& c_set(short color_pair_number);
	WINPLUS& astandend(); // plans to substitute these two for astand(bool standout);
	WINPLUS& astandout();
	WINPLUS& a_get(attr_t& attrs, short& pair, void *opts);
	WINPLUS& a_get(attr_t& attrs, short& pair);
	WINPLUS& a_off(attr_t attrs, void *opts);
	WINPLUS& a_off(attr_t attrs);
	WINPLUS& a_on(attr_t attrs, void *opts);
	WINPLUS& a_on(attr_t attrs);
	WINPLUS& a_set(attr_t attrs, short pair, void *opts);
	WINPLUS& a_set(attr_t attrs, short pair);
	WINPLUS& chga(int n, attr_t attr, short color, const void *opts);
	WINPLUS& chga(int n, attr_t attr, short color);
	
	//refresh
	WINPLUS& fresh();
};

class WPLUSMAIN: public WINPLUS
{
public:
	WPLUSMAIN();
	~WPLUSMAIN();
};

bool WINPLUS::good()
{
	return lastoper;
}

int WINPLUS::scanned()
{
	return lastscan;
}


// refresh

WINPLUS& WINPLUS::fresh()
{
	lastoper=(wrefresh(W)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::noutfresh(){
    lastoper=(wnoutrefresh( W )!=ERR);
    return *this;
}






// a-

WINPLUS& WINPLUS::aoff(int attrs)
{
	lastoper=(wattroff(W,attrs)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::aon(int attrs)
{
	lastoper=(wattron(W,attrs)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::aset(int attrs)
{
	lastoper=(wattrset(W,attrs)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::c_set(short color_pair_number, void* opts)
{
	lastoper=(wcolor_set(W,color_pair_number, opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::c_set(short color_pair_number)
{
	lastoper=(wcolor_set(W,color_pair_number, NULL)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::astandend()
{
	lastoper=(wstandend(W)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::astandout()
{
	lastoper=(wstandout(W)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_get(attr_t& attrs, short& pair, void *opts)
{
	lastoper=(wattr_get(W, &attrs, &pair, opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_get(attr_t& attrs, short& pair)
{
	lastoper=(wattr_get(W, &attrs, &pair, NULL)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_off(attr_t attrs, void *opts)
{
	lastoper=(wattr_off(W,attrs,opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_off(attr_t attrs)
{
	lastoper=(wattr_off(W,attrs,NULL)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_on(attr_t attrs, void *opts)
{
	lastoper=(wattr_on(W,attrs,opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_on(attr_t attrs)
{
	lastoper=(wattr_on(W,attrs,NULL)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_set(attr_t attrs, short pair, void *opts)
{
	lastoper=(wattr_set(W,attrs,pair,opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::a_set(attr_t attrs, short pair)
{
	lastoper=(wattr_set(W,attrs,pair,NULL)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::chga(int n, attr_t attr, short color, const void *opts)
{
	lastoper=(wchgat(W,n,attr,color,opts)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::chga(int n, attr_t attr, short color)
{
	lastoper=(wchgat(W,n,attr,color,NULL)!=ERR);
	return *this;
}








int WINPLUS::chget()
{
	return wgetch(W);
}




// std::cin & std::cout -like
WINPLUS& WINPLUS::operator<<(int k)
{
	print("%d",k);
	return *this;
}

WINPLUS& WINPLUS::operator<<(float k)
{
	print("%f",k);
	return *this;
}

WINPLUS& WINPLUS::operator<<(char k)
{
	print("%c",k);
	return *this;
}

WINPLUS& WINPLUS::operator<<(double k)
{
	print("%lf",k);
	return *this;
}

WINPLUS& WINPLUS::operator<<(char* k)
{
	print("%s",k);
	return *this;
}

WINPLUS& WINPLUS::operator<<(std::string k)
{
	print("%s",k.c_str());
	return *this;
}




WINPLUS& WINPLUS::operator>>(int& k)
{
	scan("%d", &k);
	return *this;
}

WINPLUS& WINPLUS::operator>>(float& k)
{
	scan("%f",&k);
	return *this;
}

WINPLUS& WINPLUS::operator>>(char& k)
{
	scan("%c",&k);
	return *this;
}

WINPLUS& WINPLUS::operator>>(double& k)
{
	scan("%lf",&k);
	return *this;
}

WINPLUS& WINPLUS::operator>>(char*& k)
{
	scan("%s",k);
	return *this;
}

WINPLUS& WINPLUS::operator>>(std::string& k)
{
	scan("%s",k.c_str());
	return *this;
}






WINPLUS& WINPLUS::move(int y, int x)
{
	wmove(W,y,x);
	return *this;
}



// (v)wprintw & (v)wscanw -like
WINPLUS& WINPLUS::print(char *mask, ...)
{
	int res;
	va_list args;
	va_start(args,mask);
	res=vw_printw(W,mask,args);
	va_end(args);
	lastoper=(res==OK);
	return *this;
}

WINPLUS& WINPLUS::vprint(char *mask, va_list args)
{
	lastoper=(vw_printw(W,mask,args)!=ERR);
	return *this;
}

WINPLUS& WINPLUS::scan(char *mask, ...)
{
	int res;
	va_list args;
	va_start(args,mask);
	res=vw_scanw(W,mask,args);
	va_end(args);
	lastoper=(res!=ERR);
	lastscan=res;
	return *this;
}

WINPLUS& WINPLUS::vscan(char *mask, va_list args)
{
	lastoper=(res=vw_scanw(W,mask,args)!=ERR);
	return *this;
}



// class-specific, constructors, initializers, etc.

WINPLUS::WINPLUS(int nlines, int ncols, int begin_y, int begin_x)
{
	W=newwin(nlines,ncols,begin_y,begin_x);
	lastoper=(W!=NULL);
	
};

void WINPLUS::setwpointer(WINDOW *reference)
{
	W=reference;
}

WINDOW *WINPLUS::getwpointer()
{
	return W;
}

WPLUSMAIN::WPLUSMAIN()
{
	setwpointer(initscr());
	lastoper=(getwpointer()!=NULL);
}

WPLUSMAIN::~WPLUSMAIN()
{
	endwin();
}
